package system

import (
	"kpt-pasture/http/middleware"
	"net/http"
	"strconv"

	pasturePb "gitee.com/xuyiping_admin/go_proto/proto/go/backend/cow"
	operationPb "gitee.com/xuyiping_admin/go_proto/proto/go/backend/operation"
	"gitee.com/xuyiping_admin/pkg/apierr"
	"gitee.com/xuyiping_admin/pkg/ginutil"
	"gitee.com/xuyiping_admin/pkg/valid"

	"github.com/gin-gonic/gin"
)

// RoleCreateOrUpdate 添加或者更新角色
func RoleCreateOrUpdate(c *gin.Context) {
	var req pasturePb.SearchRoleRequest
	if err := ginutil.BindProto(c, &req); err != nil {
		apierr.AbortBadRequest(c, http.StatusBadRequest, err)
		return
	}

	if err := valid.ValidateStruct(&req,
		valid.Field(&req.Name, valid.Required),
	); err != nil {
		apierr.AbortBadRequest(c, http.StatusBadRequest, err)
		return
	}

	if err := middleware.Dependency(c).StoreEventHub.OpsService.SystemRoleCreateOrUpdate(c, &req); err != nil {
		apierr.ClassifiedAbort(c, err)
		return
	}
	ginutil.JSONResp(c, &operationPb.CommonOK{
		Code: http.StatusOK,
		Msg:  "ok",
		Data: &operationPb.Success{Success: true},
	})
}

// GetRoleMenu 获取角色对应的菜单
func GetRoleMenu(c *gin.Context) {
	roleIdStr := c.Param("id")
	roleId, _ := strconv.Atoi(roleIdStr)

	if err := valid.Validate(roleId, valid.Required, valid.Min(1)); err != nil {
		apierr.ClassifiedAbort(c, err)
		return
	}

	res, err := middleware.BackendOperation(c).OpsService.GetRoleMenuList(c, int64(roleId))
	if err != nil {
		apierr.ClassifiedAbort(c, err)
		return
	}

	ginutil.JSONResp(c, res)
}

// DeleteSystemRole 删除角色
func DeleteSystemRole(c *gin.Context) {
	roleIdStr := c.Param("id")
	roleId, _ := strconv.Atoi(roleIdStr)

	if err := valid.Validate(roleId, valid.Required, valid.Min(1)); err != nil {
		apierr.ClassifiedAbort(c, err)
		return
	}

	if err := middleware.BackendOperation(c).OpsService.DeleteSystemRole(c, int64(roleId)); err != nil {
		apierr.ClassifiedAbort(c, err)
		return
	}

	ginutil.JSONResp(c, &operationPb.CommonOK{
		Code: http.StatusOK,
		Msg:  "ok",
		Data: &operationPb.Success{Success: true},
	})
}

func IsShowSystemRole(c *gin.Context) {
	roleIdStr := c.Param("id")
	roleId, _ := strconv.Atoi(roleIdStr)

	if err := valid.Validate(roleId, valid.Required, valid.Min(1)); err != nil {
		apierr.ClassifiedAbort(c, err)
		return
	}

	if err := middleware.BackendOperation(c).OpsService.IsShowSystemRole(c, int64(roleId)); err != nil {
		apierr.ClassifiedAbort(c, err)
		return
	}

	ginutil.JSONResp(c, &operationPb.CommonOK{
		Code: http.StatusOK,
		Msg:  "ok",
		Data: &operationPb.Success{Success: true},
	})
}

// SearchSystemRoleList 角色列表
func SearchSystemRoleList(c *gin.Context) {
	var req pasturePb.SearchRoleRequest
	if err := ginutil.BindProto(c, &req); err != nil {
		apierr.AbortBadRequest(c, http.StatusBadRequest, err)
		return
	}

	pagination := &pasturePb.PaginationModel{
		Page:       int32(c.GetInt(middleware.Page)),
		PageSize:   int32(c.GetInt(middleware.PageSize)),
		PageOffset: int32(c.GetInt(middleware.PageOffset)),
	}

	res, err := middleware.Dependency(c).StoreEventHub.OpsService.SearchSystemRoleList(c, &req, pagination)
	if err != nil {
		apierr.ClassifiedAbort(c, err)
		return
	}
	ginutil.JSONResp(c, res)
}

func RoleMenuSave(c *gin.Context) {
	var req pasturePb.SystemRoleMenuRequest
	if err := ginutil.BindProto(c, &req); err != nil {
		apierr.AbortBadRequest(c, http.StatusBadRequest, err)
		return
	}

	if err := valid.ValidateStruct(&req,
		valid.Field(&req.RoleId, valid.Required),
		valid.Field(&req.MenuIds, valid.Required),
	); err != nil {
		apierr.AbortBadRequest(c, http.StatusBadRequest, err)
		return
	}

	if err := middleware.Dependency(c).StoreEventHub.OpsService.RoleMenuSave(c, &req); err != nil {
		apierr.ClassifiedAbort(c, err)
		return
	}
	ginutil.JSONResp(c, &operationPb.CommonOK{
		Code: http.StatusOK,
		Msg:  "ok",
		Data: &operationPb.Success{Success: true},
	})
}

func GetRoleList(c *gin.Context) {
	res, err := middleware.Dependency(c).StoreEventHub.OpsService.SystemRoleList(c)
	if err != nil {
		apierr.ClassifiedAbort(c, err)
		return
	}
	ginutil.JSONResp(c, res)
}
