package work

import (
	"net/http"
	"strconv"

	"kpt-pasture/http/middleware"

	pasturePb "gitee.com/xuyiping_admin/go_proto/proto/go/backend/cow"
	operationPb "gitee.com/xuyiping_admin/go_proto/proto/go/backend/operation"
	"gitee.com/xuyiping_admin/pkg/apierr"
	"gitee.com/xuyiping_admin/pkg/ginutil"
	"gitee.com/xuyiping_admin/pkg/valid"
	"github.com/gin-gonic/gin"
)

func OrderList(c *gin.Context) {
	var req pasturePb.SearchWorkOrderRequest
	if err := ginutil.BindProto(c, &req); err != nil {
		apierr.AbortBadRequest(c, http.StatusBadRequest, err)
		return
	}

	pagination := &pasturePb.PaginationModel{
		Page:       int32(c.GetInt(middleware.Page)),
		PageSize:   int32(c.GetInt(middleware.PageSize)),
		PageOffset: int32(c.GetInt(middleware.PageOffset)),
	}

	res, err := middleware.Dependency(c).StoreEventHub.OpsService.OrderList(c, &req, pagination)
	if err != nil {
		apierr.ClassifiedAbort(c, err)
		return
	}
	ginutil.JSONResp(c, res)
}

func OrderCreateOrUpdate(c *gin.Context) {
	var req pasturePb.WorkOrderList
	if err := ginutil.BindProto(c, &req); err != nil {
		apierr.AbortBadRequest(c, http.StatusBadRequest, err)
		return
	}

	if err := valid.ValidateStruct(&req,
		valid.Field(&req.Name, valid.Required),
		valid.Field(&req.CategoryId, valid.Required),
		valid.Field(&req.Priority, valid.Required),
		valid.Field(&req.IsShow, valid.Required),
		valid.Field(&req.ExecTime, valid.Required),
		valid.Field(&req.SubscribeUnit, valid.Required),
		valid.Field(&req.Frequency, valid.Required),
	); err != nil {
		apierr.AbortBadRequest(c, http.StatusBadRequest, err)
		return
	}

	if err := middleware.BackendOperation(c).OpsService.OrderCreateOrUpdate(c, &req); err != nil {
		apierr.ClassifiedAbort(c, err)
		return
	}

	ginutil.JSONResp(c, &operationPb.CommonOK{
		Code: http.StatusOK,
		Msg:  "ok",
		Data: &operationPb.Success{Success: true},
	})
}

func OrderIsShow(c *gin.Context) {
	workOrderIdStr := c.Param("id")
	workOrderId, _ := strconv.Atoi(workOrderIdStr)

	if err := valid.Validate(workOrderId, valid.Required, valid.Min(1)); err != nil {
		apierr.ClassifiedAbort(c, err)
		return
	}
	if err := middleware.Dependency(c).StoreEventHub.OpsService.OrderIsShow(c, int64(workOrderId)); err != nil {
		apierr.ClassifiedAbort(c, err)
		return
	}
	ginutil.JSONResp(c, &operationPb.CommonOK{
		Code: http.StatusOK,
		Msg:  "ok",
		Data: &operationPb.Success{Success: true},
	})
}

func UserOrderList(c *gin.Context) {
	statusStr := c.Param("status")
	workOrderStatus, _ := strconv.Atoi(statusStr)

	pagination := &pasturePb.PaginationModel{
		Page:       int32(c.GetInt(middleware.Page)),
		PageSize:   int32(c.GetInt(middleware.PageSize)),
		PageOffset: int32(c.GetInt(middleware.PageOffset)),
	}

	res, err := middleware.Dependency(c).StoreEventHub.OpsService.UserWorkOrderList(c, pasturePb.WorkOrderStatus_Kind(workOrderStatus), pagination)
	if err != nil {
		apierr.ClassifiedAbort(c, err)
		return
	}

	ginutil.JSONResp(c, res)
}
